
local StoneGateV6 = ...

-- Apply building asset processor
StoneGateV6:registerAssetProcessor("models/basic/basicCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Circular Tower materials
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_CIRCULAR_RED_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Materials/blueRoof", StoneGateSetPrefix .. "_CIRCULAR_BLUE_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Materials/brownRoof", StoneGateSetPrefix .. "_CIRCULAR_BROWN_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Materials/greyRoof", StoneGateSetPrefix .. "_CIRCULAR_GREY_ROOF_MATERIAL")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Materials/grey2Roof", StoneGateSetPrefix .. "_CIRCULAR_GREY2_ROOF_MATERIAL")

-- Register Circular Tower prefab nodes
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_0_Circular_RootPart", StoneGateSetPrefix .. "_CIRCULAR_ROOT_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_1_Circular_Tiling1Part", StoneGateSetPrefix .. "_CIRCULAR_TILING1_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_2_Circular_Tiling2Part", StoneGateSetPrefix .. "_CIRCULAR_TILING2_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_1_Circular_Tiling1BPart", StoneGateSetPrefix .. "_CIRCULAR_TILING1B_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_2_Circular_Tiling2BPart", StoneGateSetPrefix .. "_CIRCULAR_TILING2B_PREFAB")
StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_3_Circular_TopPart", StoneGateSetPrefix .. "_CIRCULAR_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart2(_nodePrefix)
	StoneGateV6:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	        }
	})
end

local defaultNodePrefixList2 = {
	StoneGateSetPrefix .. "_CIRCULAR_TOP",
	StoneGateSetPrefix .. "_CIRCULAR_TILING1",
	StoneGateSetPrefix .. "_CIRCULAR_TILING2",
	StoneGateSetPrefix .. "_CIRCULAR_TILING1B",
	StoneGateSetPrefix .. "_CIRCULAR_TILING2B"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList2) do
	registerDefaultBuildingPart2(nodePrefix)
end

-- Register the stone gate set circular tower root part
StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_CIRCULAR_ROOT_PART",
	Name = StoneGateSetPrefix .. "_CIRCULAR_ROOT_NAME",
	Description = StoneGateSetPrefix .. "_CIRCULAR_ROOT_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_CIRCULAR_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_CIRCULAR_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_CIRCULAR_TILING2_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_CIRCULAR_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 9,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})

-- Register the stone gate set circular tower root part
StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_CIRCULAR_ROOTB_PART",
	Name = StoneGateSetPrefix .. "_CIRCULAR_ROOTB_NAME",
	Description = StoneGateSetPrefix .. "_CIRCULAR_ROOTB_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_CIRCULAR_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_CIRCULAR_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_CIRCULAR_TILING2B_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_CIRCULAR_TILING1B_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "WOOD", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 9,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV6:configurePrefabFlagList("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_3_Circular_TopPart", { "PLATFORM" })
 
-- Register grass clearing components

StoneGateV6:registerPrefabComponent("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_0_Circular_RootPart", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 4,
        SoftRadius = 4,
        IsPermanent = true
})

-- Override building part components properties (generated by the building asset processor) to add building zones
StoneGateV6:registerPrefabComponent("models/basic/basicCircularTower.fbx/Prefab/circularTower/step_0_Circular_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZoneData = {
            DataType = "BUILDING_ZONE", 
            Polygon = polygon.createCircle( 3.3, { 0, 0 }, 16 )
        }
})

function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV6:registerAssetId("models/basic/basicCircularTower.fbx/Prefab/circularTower/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
    })

    StoneGateV6:registerPrefabComponent("models/basic/basicCircularTower.fbx/Prefab/circularTower/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 } -- For future snapping over large gate
    })
end

registerRoofPart( "CircularRoof", "CIRCULAR_ROOF" )

---- Ramparts...

-- Apply building asset processor
StoneGateV6:registerAssetProcessor("models/ramparts/rampartCircularTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV6:registerAssetId("models/ramparts/rampartCircularTower.fbx/Prefab/circularTower/step_4_Circular_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_CIRCULAR_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

    StoneGateV6:registerPrefabComponent("models/ramparts/rampartCircularTower.fbx/Prefab/circularTower/step_4_Circular_" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
	BuildingZone = { 0, 0 } -- For future snapping over large gate
    })
end

registerRampartPart( "Rampart", "RAMPART" )
registerRampartPart( "RampartB", "RAMPART_B" )
